﻿using System.Threading;

namespace Utility
{
    /// <summary>
    /// Id序号生成器，多线程安全
    /// </summary>
    public class IdGenerator
    {
        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="startSerial">起始Id序号</param>
        public IdGenerator(long startSerial = 0)
        {
            _CurSerial = 0L;
            _Locker = new SpinLock();
            StartSerial = startSerial;
            EndSerial = 0x7fffffffffffffffL;
            _CurSerial = startSerial;
        }
        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="startSerial">起始Id序号</param>
        /// <param name="endSerial">终止序号</param>
        public IdGenerator(long startSerial, long endSerial)
            : this(startSerial)
        {
            EndSerial = endSerial;
        }


        /// <summary>
        /// 当前序号
        /// </summary>
        private long _CurSerial;
        /// <summary>
        /// 锁对象
        /// </summary>
        private SpinLock _Locker;

        /// <summary>
        /// 特殊序号常量，-1
        /// </summary>
        public static readonly long SerialMinusOne = -1L;
        /// <summary>
        /// 特殊序号常量，0
        /// </summary>
        public static readonly long SerialZero = 0L;


        /// <summary>
        /// 终止序号
        /// </summary>
        public long EndSerial { get; private set; }
        /// <summary>
        /// 起始序号
        /// </summary>
        public long StartSerial { get; private set; }


        /// <summary>
        /// 生成序号，基于当前序号的下一序号递增值，步进值1
        /// </summary>
        /// <returns></returns>
        public long GenerateSerialId()
        {
            long serial = 0L;

            bool lockTaken = false;
            _Locker.Enter(ref lockTaken);
            try
            {
                serial = Interlocked.Increment(ref _CurSerial);
                if (serial < StartSerial)
                    serial = StartSerial;
                if (serial > EndSerial)
                    serial = EndSerial;
            }
            finally
            {
                _Locker.Exit(false);
            }

            return serial;
        }
    }
}
